@@ -1,6 +1,5 @@
 #![feature(test)]
 #![allow(unused_imports)]
-#![cfg_attr(feature = "assign_ops", feature(augmented_assignments))]
 
 extern crate test;
 #[macro_use(s)]
@@ -12,68 +11,19 @@ use rblas::matrix::Matrix;
 
 use ndarray::{
     OwnedArray,
-    zeros,
+    Axis,
 };
 use ndarray::{arr0, arr1, arr2};
 
 use test::black_box;
 
 #[bench]
-fn bench_std_add_shared(bench: &mut test::Bencher)
+fn map(bench: &mut test::Bencher)
 {
-    let mut a = arr2::<f32, _>(&[[1., 2., 2.],
-                              [3., 4., 4.],
-                              [3., 4., 4.],
-                              [3., 4., 4.],
-                              [5., 6., 6.]]);
-    let b = a.clone();
-    bench.iter(|| a.iadd(&b));
-}
-
-#[bench]
-fn bench_std_add_owned(bench: &mut test::Bencher)
-{
-    let a = arr2::<f32, _>(&[[1., 2., 2.],
-                              [3., 4., 4.],
-                              [3., 4., 4.],
-                              [3., 4., 4.],
-                              [5., 6., 6.]]);
-    let mut a = a.to_owned();
-    let b = a.clone();
-    bench.iter(|| a.iadd(&b));
-}
-
-#[bench]
-fn small_iter_1d(bench: &mut test::Bencher)
-{
-    let a = arr1::<f32>(&[1., 2., 2.,
-                         3., 4., 4.,
-                         3., 4., 4.,
-                         3., 4., 4.,
-                         5., 6., 6.]);
-    bench.iter(|| for &elt in a.iter() { black_box(elt); })
-}
-
-#[bench]
-fn small_iter_1d_raw(bench: &mut test::Bencher)
-{
-    let a = arr1::<f32>(&[1., 2., 2.,
-                         3., 4., 4.,
-                         3., 4., 4.,
-                         3., 4., 4.,
-                         5., 6., 6.]);
-    bench.iter(|| for &elt in a.raw_data().iter() { black_box(elt); })
-}
-
-#[bench]
-fn small_iter_2d(bench: &mut test::Bencher)
-{
-    let a = arr2::<f32, _>(&[[1., 2., 2.],
-                          [3., 4., 4.],
-                          [3., 4., 4.],
-                          [3., 4., 4.],
-                          [5., 6., 6.]]);
-    bench.iter(|| for &elt in a.iter() { black_box(elt); })
+    let a = OwnedArray::linspace(0., 127., 128).into_shape((8, 16)).unwrap();
+    bench.iter(|| {
+        a.map(|&x| 2. * x)
+    });
 }
 
 #[bench]
@@ -154,7 +104,7 @@ fn sum_2d_raw(bench: &mut test::Bencher)
 fn sum_2d_cutout(bench: &mut test::Bencher)
 {
     let a = OwnedArray::<i32, _>::zeros((66, 66));
-    let av = a.view().slice(s![1..-1, 1..-1]);
+    let av = a.slice(s![1..-1, 1..-1]);
     let a = black_box(av);
     bench.iter(|| {
         let mut sum = 0;
@@ -169,7 +119,7 @@ fn sum_2d_cutout(bench: &mut test::Bencher)
 fn sum_2d_cutout_fold(bench: &mut test::Bencher)
 {
     let a = OwnedArray::<i32, _>::zeros((66, 66));
-    let av = a.view().slice(s![1..-1, 1..-1]);
+    let av = a.slice(s![1..-1, 1..-1]);
     let a = black_box(av);
     bench.iter(|| {
         a.fold(0, |acc, elt| acc + *elt)
@@ -190,7 +140,7 @@ fn scalar_sum_2d_regular(bench: &mut test::Bencher)
 fn scalar_sum_2d_cutout(bench: &mut test::Bencher)
 {
     let a = OwnedArray::<i32, _>::zeros((66, 66));
-    let av = a.view().slice(s![1..-1, 1..-1]);
+    let av = a.slice(s![1..-1, 1..-1]);
     let a = black_box(av);
     bench.iter(|| {
         a.scalar_sum()
@@ -201,12 +151,12 @@ fn scalar_sum_2d_cutout(bench: &mut test::Bencher)
 fn sum_2d_cutout_by_row(bench: &mut test::Bencher)
 {
     let a = OwnedArray::<i32, _>::zeros((66, 66));
-    let av = a.view().slice(s![1..-1, 1..-1]);
+    let av = a.slice(s![1..-1, 1..-1]);
     let a = black_box(av);
     bench.iter(|| {
         let mut sum = 0;
         for row in 0..a.shape()[0] {
-            for &elt in a.row_iter(row) {
+            for &elt in a.row(row) {
                 sum += elt;
             }
         }
@@ -218,7 +168,7 @@ fn sum_2d_cutout_by_row(bench: &mut test::Bencher)
 fn sum_2d_cutout_outer_iter(bench: &mut test::Bencher)
 {
     let a = OwnedArray::<i32, _>::zeros((66, 66));
-    let av = a.view().slice(s![1..-1, 1..-1]);
+    let av = a.slice(s![1..-1, 1..-1]);
     let a = black_box(av);
     bench.iter(|| {
         let mut sum = 0;
@@ -255,7 +205,7 @@ fn sum_2d_transpose_by_row(bench: &mut test::Bencher)
     bench.iter(|| {
         let mut sum = 0;
         for row in 0..a.shape()[0] {
-            for &elt in a.row_iter(row) {
+            for &elt in a.row(row) {
                 sum += elt;
             }
         }
@@ -277,7 +227,7 @@ fn scalar_sum_2d_float(bench: &mut test::Bencher)
 fn scalar_sum_2d_float_cutout(bench: &mut test::Bencher)
 {
     let a = OwnedArray::<f32, _>::zeros((66, 66));
-    let av = a.view().slice(s![1..-1, 1..-1]);
+    let av = a.slice(s![1..-1, 1..-1]);
     let a = black_box(av);
     bench.iter(|| {
         a.scalar_sum()
@@ -291,11 +241,10 @@ fn add_2d_regular(bench: &mut test::Bencher)
     let b = OwnedArray::<i32, _>::zeros((64, 64));
     let bv = b.view();
     bench.iter(|| {
-        let _x = black_box(a.view_mut() + bv);
+        a.iadd(&bv);
     });
 }
 
-#[cfg(feature = "assign_ops")]
 #[bench]
 fn add_2d_assign_ops(bench: &mut test::Bencher)
 {
@@ -317,7 +266,7 @@ fn add_2d_cutout(bench: &mut test::Bencher)
     let b = OwnedArray::<i32, _>::zeros((64, 64));
     let bv = b.view();
     bench.iter(|| {
-        let _x = black_box(acut.view_mut() + bv);
+        acut.iadd(&bv);
     });
 }
 
@@ -328,7 +277,7 @@ fn add_2d_broadcast_1_to_2(bench: &mut test::Bencher)
     let b = OwnedArray::<i32, _>::zeros(64);
     let bv = b.view();
     bench.iter(|| {
-        let _x = black_box(a.view_mut() + bv);
+        a.iadd(&bv);
     });
 }
 
@@ -339,31 +288,88 @@ fn add_2d_broadcast_0_to_2(bench: &mut test::Bencher)
     let b = OwnedArray::<i32, _>::zeros(());
     let bv = b.view();
     bench.iter(|| {
-        let _x = black_box(a.view_mut() + bv);
+        a.iadd(&bv);
+    });
+}
+
+// This is for comparison with add_2d_broadcast_0_to_2
+#[bench]
+fn add_2d_0_to_2_iadd_scalar(bench: &mut test::Bencher)
+{
+    let mut a = OwnedArray::<i32, _>::zeros((64, 64));
+    let n = black_box(0);
+    bench.iter(|| {
+        a.iadd_scalar(&n);
     });
 }
 
 #[bench]
-fn assign_scalar_2d_transposed(bench: &mut test::Bencher)
+fn add_2d_transposed(bench: &mut test::Bencher)
 {
-    let mut a = arr2::<f32, _>(&[[1., 2., 2.],
-                              [3., 4., 4.],
-                              [3., 4., 4.],
-                              [3., 4., 4.],
-                              [5., 6., 6.]]);
+    let mut a = OwnedArray::<i32, _>::zeros((64, 64));
     a.swap_axes(0, 1);
-    bench.iter(|| a.assign_scalar(&3.))
+    let b = OwnedArray::<i32, _>::zeros((64, 64));
+    let bv = b.view();
+    bench.iter(|| {
+        a.iadd(&bv);
+    });
 }
 
 #[bench]
-fn assign_scalar_2d(bench: &mut test::Bencher)
+fn add_2d_f32_regular(bench: &mut test::Bencher)
 {
-    let mut a = arr2::<f32, _>(&[[1., 2., 2.],
-                              [3., 4., 4.],
-                              [3., 4., 4.],
-                              [3., 4., 4.],
-                              [5., 6., 6.]]);
-    bench.iter(|| a.assign_scalar(&3.))
+    let mut a = OwnedArray::<f32, _>::zeros((64, 64));
+    let b = OwnedArray::<f32, _>::zeros((64, 64));
+    let bv = b.view();
+    bench.iter(|| {
+        a.iadd(&bv);
+    });
+}
+
+#[cfg(feature = "rblas")]
+#[bench]
+fn add_2d_f32_blas(bench: &mut test::Bencher)
+{
+    use rblas::Axpy;
+    use rblas::attribute::Transpose;
+    use ndarray::blas::AsBlas;
+    let mut a = OwnedArray::<f32, _>::zeros((64, 64));
+    let b = OwnedArray::<f32, _>::zeros((64, 64));
+    let len = a.len();
+    let mut av = a.view_mut().into_shape(len).unwrap();
+    let bv = b.view().into_shape(len).unwrap();
+    bench.iter(|| {
+        f32::axpy(&1., &bv.bv(), &mut av.bvm());
+    });
+}
+
+#[bench]
+fn muladd_2d_f32_regular(bench: &mut test::Bencher)
+{
+    let mut av = OwnedArray::<f32, _>::zeros((64, 64));
+    let bv = OwnedArray::<f32, _>::zeros((64, 64));
+    let scalar = 3.1415926535;
+    bench.iter(|| {
+        av.zip_mut_with(&bv, |a, &b| *a += scalar * b);
+    });
+}
+
+#[cfg(feature = "rblas")]
+#[bench]
+fn muladd_2d_f32_blas(bench: &mut test::Bencher)
+{
+    use rblas::Axpy;
+    use rblas::attribute::Transpose;
+    use ndarray::blas::AsBlas;
+    let mut a = OwnedArray::<f32, _>::zeros((64, 64));
+    let b = OwnedArray::<f32, _>::zeros((64, 64));
+    let scalar = 3.1415926535;
+    let len = a.len();
+    let mut av = a.view_mut().into_shape(len).unwrap();
+    let bv = b.view().into_shape(len).unwrap();
+    bench.iter(|| {
+        f32::axpy(&scalar, &bv.bv(), &mut av.bvm());
+    });
 }
 
 #[bench]
@@ -379,7 +385,7 @@ fn assign_scalar_2d_large(bench: &mut test::Bencher)
 fn assign_scalar_2d_cutout(bench: &mut test::Bencher)
 {
     let mut a = OwnedArray::zeros((66, 66));
-    let mut a = a.slice_mut(s![1..-1, 1..-1]);
+    let a = a.slice_mut(s![1..-1, 1..-1]);
     let mut a = black_box(a);
     let s = 3.;
     bench.iter(|| a.assign_scalar(&s))
@@ -408,14 +414,14 @@ fn assign_scalar_2d_raw_large(bench: &mut test::Bencher)
 fn bench_iter_diag(bench: &mut test::Bencher)
 {
     let a = OwnedArray::<f32, _>::zeros((1024, 1024));
-    bench.iter(|| for elt in a.diag_iter() { black_box(elt); })
+    bench.iter(|| for elt in a.diag() { black_box(elt); })
 }
 
 #[bench]
 fn bench_row_iter(bench: &mut test::Bencher)
 {
     let a = OwnedArray::<f32, _>::zeros((1024, 1024));
-    let it = a.row_iter(17);
+    let it = a.row(17);
     bench.iter(|| for elt in it.clone() { black_box(elt); })
 }
 
@@ -423,23 +429,10 @@ fn bench_row_iter(bench: &mut test::Bencher)
 fn bench_col_iter(bench: &mut test::Bencher)
 {
     let a = OwnedArray::<f32, _>::zeros((1024, 1024));
-    let it = a.col_iter(17);
+    let it = a.column(17);
     bench.iter(|| for elt in it.clone() { black_box(elt); })
 }
 
-#[bench]
-fn bench_mat_mul(bench: &mut test::Bencher)
-{
-    let a = arr2::<f32, _>(&[[1., 2., 2.],
-                          [3., 4., 4.],
-                          [3., 4., 4.],
-                          [3., 4., 4.],
-                          [5., 6., 6.]]);
-    let mut at = a.clone();
-    at.swap_axes(0, 1);
-    bench.iter(|| at.mat_mul(&a));
-}
-
 #[bench]
 fn bench_mat_mul_large(bench: &mut test::Bencher)
 {
@@ -458,15 +451,15 @@ fn bench_mat_mul_rblas_large(bench: &mut test::Bencher)
     use rblas::attribute::Transpose;
     use ndarray::blas::AsBlas;
 
-    let mut a = OwnedArray::<f32, _>::zeros((64, 64));
-    let mut b = OwnedArray::<f32, _>::zeros((64, 64));
+    let a = OwnedArray::<f32, _>::zeros((64, 64));
+    let b = OwnedArray::<f32, _>::zeros((64, 64));
     let mut c = OwnedArray::<f32, _>::zeros((64, 64));
     bench.iter(|| {
         // C ← α AB + β C
         f32::gemm(&1.,
-                  Transpose::NoTrans, &a.blas(),
-                  Transpose::NoTrans, &b.blas(),
-                  &1., &mut c.blas());
+                  Transpose::NoTrans, &a.bv(),
+                  Transpose::NoTrans, &b.bv(),
+                  &1., &mut c.bvm());
     });
 }
 
@@ -483,36 +476,94 @@ fn create_iter_4d(bench: &mut test::Bencher)
     });
 }
 
-#[bench]
-fn lst_squares(bench: &mut test::Bencher)
-{
-    let xs = arr2::<f32, _>(&[[ 2.,  3.],
-                           [-2., -1.],
-                           [ 1.,  5.],
-                           [-1.,  2.]]);
-    let b = arr1(&[1., -1., 2., 1.]);
-    bench.iter(|| ndarray::linalg::least_squares(&xs, &b));
-}
-
 #[bench]
 fn bench_to_owned_n(bench: &mut test::Bencher)
 {
-    let a = zeros::<f32, _>((32, 32));
+    let a = OwnedArray::<f32, _>::zeros((32, 32));
     bench.iter(|| a.to_owned());
 }
 
 #[bench]
 fn bench_to_owned_t(bench: &mut test::Bencher)
 {
-    let mut a = zeros::<f32, _>((32, 32));
+    let mut a = OwnedArray::<f32, _>::zeros((32, 32));
     a.swap_axes(0, 1);
     bench.iter(|| a.to_owned());
 }
 
 #[bench]
-fn equality(bench: &mut test::Bencher)
+fn equality_i32(bench: &mut test::Bencher)
 {
     let a = OwnedArray::<i32, _>::zeros((64, 64));
     let b = OwnedArray::<i32, _>::zeros((64, 64));
     bench.iter(|| a == b);
 }
+
+#[bench]
+fn equality_f32(bench: &mut test::Bencher)
+{
+    let a = OwnedArray::<f32, _>::zeros((64, 64));
+    let b = OwnedArray::<f32, _>::zeros((64, 64));
+    bench.iter(|| a == b);
+}
+
+#[bench]
+fn dot_f32_16(bench: &mut test::Bencher)
+{
+    let a = OwnedArray::<f32, _>::zeros(16);
+    let b = OwnedArray::<f32, _>::zeros(16);
+    bench.iter(|| a.dot(&b));
+}
+
+#[bench]
+fn dot_f32_32(bench: &mut test::Bencher)
+{
+    let a = OwnedArray::<f32, _>::zeros(32);
+    let b = OwnedArray::<f32, _>::zeros(32);
+    bench.iter(|| a.dot(&b));
+}
+
+#[bench]
+fn dot_f32_256(bench: &mut test::Bencher)
+{
+    let a = OwnedArray::<f32, _>::zeros(256);
+    let b = OwnedArray::<f32, _>::zeros(256);
+    bench.iter(|| a.dot(&b));
+}
+
+#[bench]
+fn dot_f32_1024(bench: &mut test::Bencher)
+{
+    let av = OwnedArray::<f32, _>::zeros(1024);
+    let bv = OwnedArray::<f32, _>::zeros(1024);
+    bench.iter(|| {
+        av.dot(&bv)
+    });
+}
+
+#[bench]
+fn dot_extended(bench: &mut test::Bencher) {
+    let m = 10;
+    let n = 33;
+    let k = 10;
+    let av = OwnedArray::<f32, _>::zeros((m, n));
+    let bv = OwnedArray::<f32, _>::zeros((n, k));
+    let mut res = OwnedArray::<f32, _>::zeros((m, k));
+    // make a manual simple matrix multiply to test
+    bench.iter(|| {
+        for i in 0..m {
+            for j in 0..k {
+                unsafe {
+                    *res.uget_mut((i, j)) = av.row(i).dot(&bv.column(j));
+                }
+            }
+        }
+    })
+}
+
+#[bench]
+fn means(bench: &mut test::Bencher) {
+    let a = OwnedArray::from_iter(0..100_000i64);
+    let a = a.into_shape((100, 1000)).unwrap();
+    bench.iter(|| a.mean(Axis(0)));
+}
